'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mixin = undefined;

var _lodash = require('lodash.isobject');

var _lodash2 = _interopRequireDefault(_lodash);

var _lodash3 = require('lodash.isplainobject');

var _lodash4 = _interopRequireDefault(_lodash3);

var _lodash5 = require('lodash.isstring');

var _lodash6 = _interopRequireDefault(_lodash5);

var _implements = require('implements');

var _implements2 = _interopRequireDefault(_implements);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var eventsFormatInvalidError = new Error('events format is invalid');
var emitterInterface = ['trigger', 'listenTo', 'on', 'off', 'stopListening'];

function isEmitter(emitter) {
  return (0, _lodash2.default)(emitter) && (0, _implements2.default)(emitter, emitterInterface);
}

function proxyTriggerSingle(srcEmitter, targetEmitter, origEvent, newEvent) {
  targetEmitter.listenTo(srcEmitter, origEvent, function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    targetEmitter.trigger.apply(targetEmitter, [newEvent].concat(args));
  });
}

function proxyTriggerAll(srcEmitter, targetEmitter) {
  targetEmitter.listenTo(srcEmitter, 'all', function (event) {
    for (var _len2 = arguments.length, args = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
      args[_key2 - 1] = arguments[_key2];
    }

    targetEmitter.trigger.apply(targetEmitter, [event].concat(args));
  });
}

function _proxyTrigger(sourceEmitter, targetEmitter) {
  var events = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'all';

  if (!isEmitter(sourceEmitter)) throw new Error('source is not an emitter');
  if (!isEmitter(targetEmitter)) throw new Error('target is not an emitter');

  if (events === 'all') return proxyTriggerAll(sourceEmitter, targetEmitter);

  var eventList = Array.isArray(events) ? events : [events];
  var eventMap = new Map();

  eventList.forEach(function (eventish) {
    if ((0, _lodash6.default)(eventish)) {
      eventish.trim().split(/\s+/).forEach(function (event) {
        if (event === '') throw eventsFormatInvalidError;
        eventMap.set(event, event);
      });
    } else if ((0, _lodash4.default)(eventish)) {
      Object.keys(eventish).forEach(function (origEvent) {
        var newEvent = eventish[origEvent];

        if (!(0, _lodash6.default)(newEvent) || newEvent.trim() === '') {
          throw eventsFormatInvalidError;
        }
        eventMap.set(origEvent, newEvent);
      });
    } else {
      throw eventsFormatInvalidError;
    }
  });

  eventMap.forEach(function (newEvent, origEvent) {
    proxyTriggerSingle(sourceEmitter, targetEmitter, origEvent, newEvent);
  });
}

exports.default = _proxyTrigger;
var mixin = exports.mixin = Object.freeze({
  proxyTrigger: function proxyTrigger(sourceEmitter, events) {
    _proxyTrigger(sourceEmitter, this, events);
  }
});